package org.jboss.tools.sapphire.internal;

import org.eclipse.sapphire.modeling.IModelParticle;
import org.eclipse.sapphire.modeling.ListBindingImpl;
import org.eclipse.sapphire.modeling.ModelElement;
import org.eclipse.sapphire.modeling.ModelElementList;
import org.eclipse.sapphire.modeling.ModelProperty;
import org.eclipse.sapphire.modeling.Resource;
import org.eclipse.sapphire.modeling.Value;
import org.eclipse.sapphire.modeling.ValueProperty;
import org.jboss.tools.sapphire.IComponent;
import org.jboss.tools.sapphire.IComponentService;
import org.jboss.tools.sapphire.IComponentServiceReference;
import org.jboss.tools.sapphire.IImplementationBean;

public final class Component
    
    extends ModelElement
    implements IComponent
    
{
    private ModelElementList<IComponentServiceReference> componentServiceReferences;
    private ModelElementList<IComponentService> componentServices;
    private ModelElementList<IImplementationBean> implementationBeans;
    private Value<String> name;
    
    public Component( final IModelParticle parent, final ModelProperty parentProperty, final Resource resource )
    {
        super( TYPE, parent, parentProperty, resource );
    }
    
    public Component( final Resource resource )
    {
        super( TYPE, null, null, resource );
    }
    
    public ModelElementList<IComponentService> getComponentServices()
    {
        synchronized( root() )
        {
            if( this.componentServices == null )
            {
                refresh( PROP_COMPONENT_SERVICES, true );
            }
            
            return this.componentServices;
        }
    }
    
    public ModelElementList<IComponentServiceReference> getComponentServiceReferences()
    {
        synchronized( root() )
        {
            if( this.componentServiceReferences == null )
            {
                refresh( PROP_COMPONENT_SERVICE_REFERENCES, true );
            }
            
            return this.componentServiceReferences;
        }
    }
    
    public ModelElementList<IImplementationBean> getImplementationBeans()
    {
        synchronized( root() )
        {
            if( this.implementationBeans == null )
            {
                refresh( PROP_IMPLEMENTATION_BEANS, true );
            }
            
            return this.implementationBeans;
        }
    }
    
    public Value<String> getName()
    {
        synchronized( root() )
        {
            if( this.name == null )
            {
                refresh( PROP_NAME, true );
            }
            
            return this.name;
        }
    }
    
    public void setName( String value )
    {
        synchronized( root() )
        {
            if( value != null && value.equals( "" ) )
            {
                value = null;
            }
            
            value = PROP_NAME.decodeKeywords( value );
            
            refresh( PROP_NAME, true );
            
            if( ! equal( this.name.getText( false ), value ) )
            {
                resource().binding( PROP_NAME ).write( value );
                refresh( PROP_NAME, false );
            }
        }
    }
    
    protected void refreshProperty( ModelProperty property, final boolean force )
    {
        synchronized( root() )
        {
            property = property.refine( this );
            
            if( property == PROP_COMPONENT_SERVICES )
            {
                if( this.componentServices == null )
                {
                    if( force == true )
                    {
                        this.componentServices = new ModelElementList<IComponentService>( this, PROP_COMPONENT_SERVICES );
                        final ListBindingImpl binding = resource().binding( PROP_COMPONENT_SERVICES );
                        this.componentServices.init( binding );
                        refreshPropertyEnabledStatus( PROP_COMPONENT_SERVICES );
                    }
                }
                else
                {
                    final boolean propertyEnabledStatusChanged = refreshPropertyEnabledStatus( PROP_COMPONENT_SERVICES );
                    final boolean notified = this.componentServices.refresh();
                    
                    if( ! notified && propertyEnabledStatusChanged )
                    {
                        notifyPropertyChangeListeners( PROP_COMPONENT_SERVICES );
                    }
                }
            }
            else if( property == PROP_COMPONENT_SERVICE_REFERENCES )
            {
                if( this.componentServiceReferences == null )
                {
                    if( force == true )
                    {
                        this.componentServiceReferences = new ModelElementList<IComponentServiceReference>( this, PROP_COMPONENT_SERVICE_REFERENCES );
                        final ListBindingImpl binding = resource().binding( PROP_COMPONENT_SERVICE_REFERENCES );
                        this.componentServiceReferences.init( binding );
                        refreshPropertyEnabledStatus( PROP_COMPONENT_SERVICE_REFERENCES );
                    }
                }
                else
                {
                    final boolean propertyEnabledStatusChanged = refreshPropertyEnabledStatus( PROP_COMPONENT_SERVICE_REFERENCES );
                    final boolean notified = this.componentServiceReferences.refresh();
                    
                    if( ! notified && propertyEnabledStatusChanged )
                    {
                        notifyPropertyChangeListeners( PROP_COMPONENT_SERVICE_REFERENCES );
                    }
                }
            }
            else if( property == PROP_IMPLEMENTATION_BEANS )
            {
                if( this.implementationBeans == null )
                {
                    if( force == true )
                    {
                        this.implementationBeans = new ModelElementList<IImplementationBean>( this, PROP_IMPLEMENTATION_BEANS );
                        final ListBindingImpl binding = resource().binding( PROP_IMPLEMENTATION_BEANS );
                        this.implementationBeans.init( binding );
                        refreshPropertyEnabledStatus( PROP_IMPLEMENTATION_BEANS );
                    }
                }
                else
                {
                    final boolean propertyEnabledStatusChanged = refreshPropertyEnabledStatus( PROP_IMPLEMENTATION_BEANS );
                    final boolean notified = this.implementationBeans.refresh();
                    
                    if( ! notified && propertyEnabledStatusChanged )
                    {
                        notifyPropertyChangeListeners( PROP_IMPLEMENTATION_BEANS );
                    }
                }
            }
            else if( property == PROP_NAME )
            {
                if( this.name != null || force == true )
                {
                    final Value<String> oldValue = this.name;
                    
                    final String val = resource().binding( PROP_NAME ).read();
                    
                    this.name = new Value<String>( this, PROP_NAME, PROP_NAME.encodeKeywords( val ) );
                    this.name.init();
                    
                    final boolean propertyEnabledStatusChanged = refreshPropertyEnabledStatus( PROP_NAME );
                    
                    if( oldValue != null )
                    {
                        if( this.name.equals( oldValue ) )
                        {
                            this.name = oldValue;
                        }
                        
                        if( this.name != oldValue || propertyEnabledStatusChanged )
                        {
                            notifyPropertyChangeListeners( PROP_NAME );
                        }
                    }
                }
            }
        }
    }
    
    public Object read( ModelProperty property )
    {
        property = property.refine( this );
        
        if( property == PROP_COMPONENT_SERVICES )
        {
            return getComponentServices();
        }
        else if( property == PROP_COMPONENT_SERVICE_REFERENCES )
        {
            return getComponentServiceReferences();
        }
        else if( property == PROP_IMPLEMENTATION_BEANS )
        {
            return getImplementationBeans();
        }
        else if( property == PROP_NAME )
        {
            return getName();
        }
        
        return super.read( property );
    }
    
    public void write( ValueProperty property, final Object value )
    {
        property = (ValueProperty) property.refine( this );
        
        if( property == PROP_NAME )
        {
            setName( (String) value );
            return;
        }
        
        super.write( property, value );
    }
    
}
