package org.jboss.tools.sapphire.internal;

import org.eclipse.sapphire.modeling.IModelParticle;
import org.eclipse.sapphire.modeling.ListBindingImpl;
import org.eclipse.sapphire.modeling.ModelElement;
import org.eclipse.sapphire.modeling.ModelElementList;
import org.eclipse.sapphire.modeling.ModelProperty;
import org.eclipse.sapphire.modeling.Resource;
import org.eclipse.sapphire.modeling.Value;
import org.eclipse.sapphire.modeling.ValueProperty;
import org.jboss.tools.sapphire.IComponentService;
import org.jboss.tools.sapphire.IInterfaceJava;

public final class ComponentService
    
    extends ModelElement
    implements IComponentService
    
{
    private ModelElementList<IInterfaceJava> javaInterfaces;
    private Value<String> name;
    
    public ComponentService( final IModelParticle parent, final ModelProperty parentProperty, final Resource resource )
    {
        super( TYPE, parent, parentProperty, resource );
    }
    
    public ComponentService( final Resource resource )
    {
        super( TYPE, null, null, resource );
    }
    
    public ModelElementList<IInterfaceJava> getJavaInterfaces()
    {
        synchronized( root() )
        {
            if( this.javaInterfaces == null )
            {
                refresh( PROP_JAVA_INTERFACES, true );
            }
            
            return this.javaInterfaces;
        }
    }
    
    public Value<String> getName()
    {
        synchronized( root() )
        {
            if( this.name == null )
            {
                refresh( PROP_NAME, true );
            }
            
            return this.name;
        }
    }
    
    public void setName( String value )
    {
        synchronized( root() )
        {
            if( value != null && value.equals( "" ) )
            {
                value = null;
            }
            
            value = PROP_NAME.decodeKeywords( value );
            
            refresh( PROP_NAME, true );
            
            if( ! equal( this.name.getText( false ), value ) )
            {
                resource().binding( PROP_NAME ).write( value );
                refresh( PROP_NAME, false );
            }
        }
    }
    
    protected void refreshProperty( ModelProperty property, final boolean force )
    {
        synchronized( root() )
        {
            property = property.refine( this );
            
            if( property == PROP_JAVA_INTERFACES )
            {
                if( this.javaInterfaces == null )
                {
                    if( force == true )
                    {
                        this.javaInterfaces = new ModelElementList<IInterfaceJava>( this, PROP_JAVA_INTERFACES );
                        final ListBindingImpl binding = resource().binding( PROP_JAVA_INTERFACES );
                        this.javaInterfaces.init( binding );
                        refreshPropertyEnabledStatus( PROP_JAVA_INTERFACES );
                    }
                }
                else
                {
                    final boolean propertyEnabledStatusChanged = refreshPropertyEnabledStatus( PROP_JAVA_INTERFACES );
                    final boolean notified = this.javaInterfaces.refresh();
                    
                    if( ! notified && propertyEnabledStatusChanged )
                    {
                        notifyPropertyChangeListeners( PROP_JAVA_INTERFACES );
                    }
                }
            }
            else if( property == PROP_NAME )
            {
                if( this.name != null || force == true )
                {
                    final Value<String> oldValue = this.name;
                    
                    final String val = resource().binding( PROP_NAME ).read();
                    
                    this.name = new Value<String>( this, PROP_NAME, PROP_NAME.encodeKeywords( val ) );
                    this.name.init();
                    
                    final boolean propertyEnabledStatusChanged = refreshPropertyEnabledStatus( PROP_NAME );
                    
                    if( oldValue != null )
                    {
                        if( this.name.equals( oldValue ) )
                        {
                            this.name = oldValue;
                        }
                        
                        if( this.name != oldValue || propertyEnabledStatusChanged )
                        {
                            notifyPropertyChangeListeners( PROP_NAME );
                        }
                    }
                }
            }
        }
    }
    
    public Object read( ModelProperty property )
    {
        property = property.refine( this );
        
        if( property == PROP_JAVA_INTERFACES )
        {
            return getJavaInterfaces();
        }
        else if( property == PROP_NAME )
        {
            return getName();
        }
        
        return super.read( property );
    }
    
    public void write( ValueProperty property, final Object value )
    {
        property = (ValueProperty) property.refine( this );
        
        if( property == PROP_NAME )
        {
            setName( (String) value );
            return;
        }
        
        super.write( property, value );
    }
    
}
