package org.jboss.tools.sapphire.internal;

import org.eclipse.sapphire.modeling.IModelParticle;
import org.eclipse.sapphire.modeling.ListBindingImpl;
import org.eclipse.sapphire.modeling.ModelElement;
import org.eclipse.sapphire.modeling.ModelElementList;
import org.eclipse.sapphire.modeling.ModelProperty;
import org.eclipse.sapphire.modeling.Resource;
import org.eclipse.sapphire.modeling.Value;
import org.eclipse.sapphire.modeling.ValueProperty;
import org.jboss.tools.sapphire.IComponent;
import org.jboss.tools.sapphire.IComposite;
import org.jboss.tools.sapphire.IService;

public final class Composite
    
    extends ModelElement
    implements IComposite
    
{
    private ModelElementList<IComponent> components;
    private Value<String> name;
    private ModelElementList<IService> services;
    private Value<String> targetNamespace;
    
    public Composite( final IModelParticle parent, final ModelProperty parentProperty, final Resource resource )
    {
        super( TYPE, parent, parentProperty, resource );
    }
    
    public Composite( final Resource resource )
    {
        super( TYPE, null, null, resource );
    }
    
    public ModelElementList<IComponent> getComponents()
    {
        synchronized( root() )
        {
            if( this.components == null )
            {
                refresh( PROP_COMPONENTS, true );
            }
            
            return this.components;
        }
    }
    
    public Value<String> getName()
    {
        synchronized( root() )
        {
            if( this.name == null )
            {
                refresh( PROP_NAME, true );
            }
            
            return this.name;
        }
    }
    
    public void setName( String value )
    {
        synchronized( root() )
        {
            if( value != null && value.equals( "" ) )
            {
                value = null;
            }
            
            value = PROP_NAME.decodeKeywords( value );
            
            refresh( PROP_NAME, true );
            
            if( ! equal( this.name.getText( false ), value ) )
            {
                resource().binding( PROP_NAME ).write( value );
                refresh( PROP_NAME, false );
            }
        }
    }
    
    public ModelElementList<IService> getServices()
    {
        synchronized( root() )
        {
            if( this.services == null )
            {
                refresh( PROP_SERVICES, true );
            }
            
            return this.services;
        }
    }
    
    public Value<String> getTargetNamespace()
    {
        synchronized( root() )
        {
            if( this.targetNamespace == null )
            {
                refresh( PROP_TARGET_NAMESPACE, true );
            }
            
            return this.targetNamespace;
        }
    }
    
    public void setTargetNamespace( String value )
    {
        synchronized( root() )
        {
            if( value != null && value.equals( "" ) )
            {
                value = null;
            }
            
            value = PROP_TARGET_NAMESPACE.decodeKeywords( value );
            
            refresh( PROP_TARGET_NAMESPACE, true );
            
            if( ! equal( this.targetNamespace.getText( false ), value ) )
            {
                resource().binding( PROP_TARGET_NAMESPACE ).write( value );
                refresh( PROP_TARGET_NAMESPACE, false );
            }
        }
    }
    
    protected void refreshProperty( ModelProperty property, final boolean force )
    {
        synchronized( root() )
        {
            property = property.refine( this );
            
            if( property == PROP_COMPONENTS )
            {
                if( this.components == null )
                {
                    if( force == true )
                    {
                        this.components = new ModelElementList<IComponent>( this, PROP_COMPONENTS );
                        final ListBindingImpl binding = resource().binding( PROP_COMPONENTS );
                        this.components.init( binding );
                        refreshPropertyEnabledStatus( PROP_COMPONENTS );
                    }
                }
                else
                {
                    final boolean propertyEnabledStatusChanged = refreshPropertyEnabledStatus( PROP_COMPONENTS );
                    final boolean notified = this.components.refresh();
                    
                    if( ! notified && propertyEnabledStatusChanged )
                    {
                        notifyPropertyChangeListeners( PROP_COMPONENTS );
                    }
                }
            }
            else if( property == PROP_NAME )
            {
                if( this.name != null || force == true )
                {
                    final Value<String> oldValue = this.name;
                    
                    final String val = resource().binding( PROP_NAME ).read();
                    
                    this.name = new Value<String>( this, PROP_NAME, PROP_NAME.encodeKeywords( val ) );
                    this.name.init();
                    
                    final boolean propertyEnabledStatusChanged = refreshPropertyEnabledStatus( PROP_NAME );
                    
                    if( oldValue != null )
                    {
                        if( this.name.equals( oldValue ) )
                        {
                            this.name = oldValue;
                        }
                        
                        if( this.name != oldValue || propertyEnabledStatusChanged )
                        {
                            notifyPropertyChangeListeners( PROP_NAME );
                        }
                    }
                }
            }
            else if( property == PROP_SERVICES )
            {
                if( this.services == null )
                {
                    if( force == true )
                    {
                        this.services = new ModelElementList<IService>( this, PROP_SERVICES );
                        final ListBindingImpl binding = resource().binding( PROP_SERVICES );
                        this.services.init( binding );
                        refreshPropertyEnabledStatus( PROP_SERVICES );
                    }
                }
                else
                {
                    final boolean propertyEnabledStatusChanged = refreshPropertyEnabledStatus( PROP_SERVICES );
                    final boolean notified = this.services.refresh();
                    
                    if( ! notified && propertyEnabledStatusChanged )
                    {
                        notifyPropertyChangeListeners( PROP_SERVICES );
                    }
                }
            }
            else if( property == PROP_TARGET_NAMESPACE )
            {
                if( this.targetNamespace != null || force == true )
                {
                    final Value<String> oldValue = this.targetNamespace;
                    
                    final String val = resource().binding( PROP_TARGET_NAMESPACE ).read();
                    
                    this.targetNamespace = new Value<String>( this, PROP_TARGET_NAMESPACE, PROP_TARGET_NAMESPACE.encodeKeywords( val ) );
                    this.targetNamespace.init();
                    
                    final boolean propertyEnabledStatusChanged = refreshPropertyEnabledStatus( PROP_TARGET_NAMESPACE );
                    
                    if( oldValue != null )
                    {
                        if( this.targetNamespace.equals( oldValue ) )
                        {
                            this.targetNamespace = oldValue;
                        }
                        
                        if( this.targetNamespace != oldValue || propertyEnabledStatusChanged )
                        {
                            notifyPropertyChangeListeners( PROP_TARGET_NAMESPACE );
                        }
                    }
                }
            }
        }
    }
    
    public Object read( ModelProperty property )
    {
        property = property.refine( this );
        
        if( property == PROP_COMPONENTS )
        {
            return getComponents();
        }
        else if( property == PROP_NAME )
        {
            return getName();
        }
        else if( property == PROP_SERVICES )
        {
            return getServices();
        }
        else if( property == PROP_TARGET_NAMESPACE )
        {
            return getTargetNamespace();
        }
        
        return super.read( property );
    }
    
    public void write( ValueProperty property, final Object value )
    {
        property = (ValueProperty) property.refine( this );
        
        if( property == PROP_NAME )
        {
            setName( (String) value );
            return;
        }
        else if( property == PROP_TARGET_NAMESPACE )
        {
            setTargetNamespace( (String) value );
            return;
        }
        
        super.write( property, value );
    }
    
}
