package org.jboss.tools.sapphire.internal;

import org.eclipse.sapphire.modeling.IModelParticle;
import org.eclipse.sapphire.modeling.ModelElement;
import org.eclipse.sapphire.modeling.ModelProperty;
import org.eclipse.sapphire.modeling.Resource;
import org.eclipse.sapphire.modeling.Value;
import org.eclipse.sapphire.modeling.ValueProperty;
import org.jboss.tools.sapphire.IImplementationBean;

public final class ImplementationBean
    
    extends ModelElement
    implements IImplementationBean
    
{
    private Value<String> implBeanClass;
    
    public ImplementationBean( final IModelParticle parent, final ModelProperty parentProperty, final Resource resource )
    {
        super( TYPE, parent, parentProperty, resource );
    }
    
    public ImplementationBean( final Resource resource )
    {
        super( TYPE, null, null, resource );
    }
    
    public Value<String> getImplBeanClass()
    {
        synchronized( root() )
        {
            if( this.implBeanClass == null )
            {
                refresh( PROP_IMPL_BEAN_CLASS, true );
            }
            
            return this.implBeanClass;
        }
    }
    
    public void setImplBeanClass( String value )
    {
        synchronized( root() )
        {
            if( value != null && value.equals( "" ) )
            {
                value = null;
            }
            
            value = PROP_IMPL_BEAN_CLASS.decodeKeywords( value );
            
            refresh( PROP_IMPL_BEAN_CLASS, true );
            
            if( ! equal( this.implBeanClass.getText( false ), value ) )
            {
                resource().binding( PROP_IMPL_BEAN_CLASS ).write( value );
                refresh( PROP_IMPL_BEAN_CLASS, false );
            }
        }
    }
    
    protected void refreshProperty( ModelProperty property, final boolean force )
    {
        synchronized( root() )
        {
            property = property.refine( this );
            
            if( property == PROP_IMPL_BEAN_CLASS )
            {
                if( this.implBeanClass != null || force == true )
                {
                    final Value<String> oldValue = this.implBeanClass;
                    
                    final String val = resource().binding( PROP_IMPL_BEAN_CLASS ).read();
                    
                    this.implBeanClass = new Value<String>( this, PROP_IMPL_BEAN_CLASS, PROP_IMPL_BEAN_CLASS.encodeKeywords( val ) );
                    this.implBeanClass.init();
                    
                    final boolean propertyEnabledStatusChanged = refreshPropertyEnabledStatus( PROP_IMPL_BEAN_CLASS );
                    
                    if( oldValue != null )
                    {
                        if( this.implBeanClass.equals( oldValue ) )
                        {
                            this.implBeanClass = oldValue;
                        }
                        
                        if( this.implBeanClass != oldValue || propertyEnabledStatusChanged )
                        {
                            notifyPropertyChangeListeners( PROP_IMPL_BEAN_CLASS );
                        }
                    }
                }
            }
        }
    }
    
    public Object read( ModelProperty property )
    {
        property = property.refine( this );
        
        if( property == PROP_IMPL_BEAN_CLASS )
        {
            return getImplBeanClass();
        }
        
        return super.read( property );
    }
    
    public void write( ValueProperty property, final Object value )
    {
        property = (ValueProperty) property.refine( this );
        
        if( property == PROP_IMPL_BEAN_CLASS )
        {
            setImplBeanClass( (String) value );
            return;
        }
        
        super.write( property, value );
    }
    
}
