package org.jboss.tools.sapphire.internal;

import org.eclipse.sapphire.modeling.IModelParticle;
import org.eclipse.sapphire.modeling.ModelElement;
import org.eclipse.sapphire.modeling.ModelProperty;
import org.eclipse.sapphire.modeling.Resource;
import org.eclipse.sapphire.modeling.Value;
import org.eclipse.sapphire.modeling.ValueProperty;
import org.jboss.tools.sapphire.IInterfaceJava;

public final class InterfaceJava
    
    extends ModelElement
    implements IInterfaceJava
    
{
    private Value<String> javaInterface;
    
    public InterfaceJava( final IModelParticle parent, final ModelProperty parentProperty, final Resource resource )
    {
        super( TYPE, parent, parentProperty, resource );
    }
    
    public InterfaceJava( final Resource resource )
    {
        super( TYPE, null, null, resource );
    }
    
    public Value<String> getJavaInterface()
    {
        synchronized( root() )
        {
            if( this.javaInterface == null )
            {
                refresh( PROP_JAVA_INTERFACE, true );
            }
            
            return this.javaInterface;
        }
    }
    
    public void setJavaInterface( String value )
    {
        synchronized( root() )
        {
            if( value != null && value.equals( "" ) )
            {
                value = null;
            }
            
            value = PROP_JAVA_INTERFACE.decodeKeywords( value );
            
            refresh( PROP_JAVA_INTERFACE, true );
            
            if( ! equal( this.javaInterface.getText( false ), value ) )
            {
                resource().binding( PROP_JAVA_INTERFACE ).write( value );
                refresh( PROP_JAVA_INTERFACE, false );
            }
        }
    }
    
    protected void refreshProperty( ModelProperty property, final boolean force )
    {
        synchronized( root() )
        {
            property = property.refine( this );
            
            if( property == PROP_JAVA_INTERFACE )
            {
                if( this.javaInterface != null || force == true )
                {
                    final Value<String> oldValue = this.javaInterface;
                    
                    final String val = resource().binding( PROP_JAVA_INTERFACE ).read();
                    
                    this.javaInterface = new Value<String>( this, PROP_JAVA_INTERFACE, PROP_JAVA_INTERFACE.encodeKeywords( val ) );
                    this.javaInterface.init();
                    
                    final boolean propertyEnabledStatusChanged = refreshPropertyEnabledStatus( PROP_JAVA_INTERFACE );
                    
                    if( oldValue != null )
                    {
                        if( this.javaInterface.equals( oldValue ) )
                        {
                            this.javaInterface = oldValue;
                        }
                        
                        if( this.javaInterface != oldValue || propertyEnabledStatusChanged )
                        {
                            notifyPropertyChangeListeners( PROP_JAVA_INTERFACE );
                        }
                    }
                }
            }
        }
    }
    
    public Object read( ModelProperty property )
    {
        property = property.refine( this );
        
        if( property == PROP_JAVA_INTERFACE )
        {
            return getJavaInterface();
        }
        
        return super.read( property );
    }
    
    public void write( ValueProperty property, final Object value )
    {
        property = (ValueProperty) property.refine( this );
        
        if( property == PROP_JAVA_INTERFACE )
        {
            setJavaInterface( (String) value );
            return;
        }
        
        super.write( property, value );
    }
    
}
